package testcases.scss.util;

import java.awt.Image;
import java.awt.image.BufferedImage;

/**
 * Default image creation strategy.
 *
 *  Represents the default strategy for creating images (java.awt.Image)
 *  from byte arrays. This strategy uses a java.awt.image.BufferedImage object.
 *
 * @author Dennis Wagelaar
 * @version 23 October 2002
 */

public class DefaultImageCreationStrategy implements ImageCreationStrategy {

	/**
	 * Creates an Image from a byte array. Assumes a grayscale
	 * image (8 bpp) of given width and height.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param image The byte array containing the pixels
	 * @return The image object
	 */
	public Image createBWImage(int width, int height, byte[] image) {
		int pixel;
		int[] pixels = new int[width * height];
		BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		// create 32 bit integer array of rgb pixels (r=g=b)
		for (int i = 0; i < width * height; i++) {
			pixel = image[i];
			pixel &= 0xFF;
			pixels[i] = (255 << 24) | (pixel << 16) | (pixel << 8) | pixel;
		}
		// draw onto buffered image and return
		bi.setRGB(0, 0, width, height, pixels, 0, width);
		return bi;
	} //createBWImage

	/**
	 * Creates an Image from a byte array. Assumes an RGB
	 * image (24 bpp) of given width and height.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param image The byte array containing the pixels
	 * @return The image object
	 */
	public Image createRGBImage(int width, int height, byte[] image) {
		int r, g, b;
		int[] pixels = new int[width * height];
		BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		// create 32 bit integer array of rgb pixels
		for (int i = 0; i < width * height; i++) {
			r = image[i * 3];
			g = image[i * 3 + 1];
			b = image[i * 3 + 2];
			r &= 0xFF;
			g &= 0xFF;
			b &= 0xFF;
			pixels[i] = (255 << 24) | (r << 16) | (g << 8) | b;
		}
		// draw onto buffered image and return
		bi.setRGB(0, 0, width, height, pixels, 0, width);
		return bi;
	} //createRGBImage

} //class DefaultImageCreationStrategy
