package testcases.scss.util;

import java.awt.Image;
import javax.swing.ImageIcon;

/**
 * Video image class returned by the StorageController.
 *
 *  This class contains all data on a video image stored in the
 *  database. It stores a date/time in a precision of milliseconds,
 *  a camera ID and an image.
 *
 * @author Miro Casanova
 * @author Dennis Wagelaar
 * @author Peter Rigole
 * @version 28 October 2002
 */

public class StorageVideoImage extends TimeData {

	////////////////////////////////////////////////////////////////////////
	// Attributes
	////////////////////////////////////////////////////////////////////////

	private ID $camid = null;
	private ImageIcon $image = null; //new ImageIcon(); // this one is serializable
	private ImageCreationStrategy $imageCreation =
		new DefaultImageCreationStrategy(); // creates Image from byte[]
	/* preliminary image attributes */
	private byte[] $bImage = null;
	private boolean $rgb = false;
	private int $width;
	private int $height;

	////////////////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////////////////

	/**
	 * Constructs a StorageVideoImage.
	 * @author Miro Casanova
	 */
	public StorageVideoImage() {
		super();
	} //StorageVideoImage

	/**
	 * Constructs a StorageVideoImage.
	 * @param time The time data for the image
	 * @param camid The camera ID
	 * @param image The image data
	 * @author Dennis Wagelaar
	 */

	public StorageVideoImage(TimeData time, ID camid, Image image) {
		super(time);
		this.setCameraID(camid);
		this.setImage(image);
	} //StorageVideoImage


	/**
	 * Constructs a StorageVideoImage.
	 * @param year The year (e.g. 2002)
	 * @param month The month (e.g. 12)
	 * @param day The day of the month (e.g. 31)
	 * @param hour The hour (e.g. 23)
	 * @param minute The minute (e.g. 59)
	 * @param second The second (e.g. 59)
	 * @param msecond The amount of milliseconds (e.g. 999)
	 * @param camid The camera ID
	 * @param image The image data
	 * @author Dennis Wagelaar
	 */

	public StorageVideoImage(int year, int month, int day, int hour, int minute,
			int second, int msecond, ID camid, Image image) {
		super(year, month, day, hour, minute, second, msecond);
		this.setCameraID(camid);
		this.setImage(image);
	} //StorageVideoImage


	////////////////////////////////////////////////////////////////////////
	// Public methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Sets the strategy for creating an Image from byte[].
	 * @param ics The ImageCreationStrategy to use
	 * @throws NullPointerException if ics is null
	 * @author Dennis Wagelaar
	 */
	public void setImageCreationStrategy(ImageCreationStrategy ics)
			throws NullPointerException {
		if (ics != null) {
			this.$imageCreation = ics;
		} else {
			throw new NullPointerException("Image creation strategy may not be null");
		}
	} //setImageCreationStrategy

	/**
	 * Sets the image data attribute from a byte array.
	 * @param width The width of the image in pixels
	 * @param height The height of the image in pixels
	 * @param rgb If true, assumes a 24 bpp RGB image, otherwise an
	 *        8 bpp grayscale image
	 * @param image The byte array of pixels
	 * @author Dennis Wagelaar
	 * @author Peter Rigole
	 */
	public void setImage(int width, int height, boolean rgb, byte[] image) {
		// postpone Image creation and save preliminary image data
		$bImage = image;
		$rgb = rgb;
		$width = width;
		$height = height;
		$image = null;
		/*
		Image img;
		if (rgb) {
			img = this.$imageCreation.createRGBImage(width, height, image);
		} else {
			img = this.$imageCreation.createBWImage(width, height, image);
		}
		this.setImage(img);
		*/
	} //setImage

	/**
	 * Sets the image data attribute.
	 * @param i The image data
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 * @author Peter Rigole
	 */
	public void setImage(Image i) {
		//this.$image.setImage(i);
		this.$image = new ImageIcon(i);
	} //setImage

	/**
	 * Gets the image data attribute.
	 * @return The image data
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 * @author Peter Rigole
	 */
	public Image getImage() {
		//return $image.getImage();
		// we now create the Image only when it is requested,
		// if there is no Image object stored:
		if ($image == null) {
			if ($bImage != null) {
				if ($rgb) {
					this.setImage($imageCreation.createRGBImage(
						$width, $height, $bImage));
				} else {
					this.setImage($imageCreation.createBWImage(
						$width, $height, $bImage));
				}
			} else {
				return null;
			}
		}
		return $image.getImage();
	} //getImage

	/**
	 * Sets the camera ID attribute.
	 * @param camid The camera ID
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public void setCameraID(ID camid) {
		this.$camid = camid;
	} //setCameraID

	/**
	 * Gets the camera ID attribute.
	 * @return The camera ID
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public ID getCameraID() {
		return this.$camid;
	} //getCameraID

	/**
	 * Generates an ID for this object from the camera ID attribute
	 * and the timestamp.
	 * @return The ID for this object
	 * @author Dennis Wagelaar
	 */
	public ID getID() {
		return new ID(this.getCameraID().getID() + "-" +
			this.getTimestamp() + "-" + this.getMilliSec());
	} //getID

} //class StorageVideoImage
