package testcases.scss.util;

import java.io.Serializable;

import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;

import java.text.SimpleDateFormat;

/**
 * Time/date class for SEESCOA.
 *
 *  TimeData stores a date/time in a precision of milliseconds.
 *
 * @author Miro Casanova
 * @author Dennis Wagelaar
 * @version 23 October 2002
 */

public class TimeData implements Serializable {

	////////////////////////////////////////////////////////////////////////
	// Attributes
	////////////////////////////////////////////////////////////////////////

	private GregorianCalendar $calendar;
	private int $millisec;
	private SimpleDateFormat $timeFmt =
		new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	private SimpleDateFormat $timestampFmt =
		new SimpleDateFormat("yyyyMMddHHmmss");

	////////////////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////////////////

	/**
	 * Constructs a TimeData and initialises to the current time.
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public TimeData() {
		$calendar = new GregorianCalendar();
		$millisec = $calendar.get(Calendar.MILLISECOND);
	} //TimeData

	/**
	 * Constructs a TimeData.
	 * @param time The time data to copy from
	 * @author Dennis Wagelaar
	 */
	public TimeData(TimeData time) {
		this.setTime(time);
	} //TimeData

	/**
	 * Constructs a TimeData.
	 * @param year The year (e.g. 2002)
	 * @param month The month (e.g. 12)
	 * @param day The day of the month (e.g. 31)
	 * @param hour The hour (e.g. 23)
	 * @param minute The minute (e.g. 59)
	 * @param second The second (e.g. 59)
	 * @param msecond The amount of milliseconds (e.g. 999)
	 * @author Miro Casanova
	 * @author Dennis Wagelaar
	 */
	public TimeData(int year, int month, int day, int hour, int minute, int second,
			int msecond) {
		// Month value is 0-based
		$calendar = new GregorianCalendar(year, month - 1, day, hour, minute,
			second);
		$millisec = msecond;
	} //TimeData

	////////////////////////////////////////////////////////////////////////
	// Public methods
	////////////////////////////////////////////////////////////////////////

	/**
	 * Sets the time attribute using a Date type. Note that the millisecond
	 * part is not included.
	 * @param time The date/time
	 * @author Dennis Wagelaar
	 */
	public void setTimeAsDate(Date time)
	{
		this.$calendar.setTime(time);
	} //setTimeAsDate

	/**
	 * Gets the time attribute as a Date type. Note that the millisecond
	 * part is not included.
	 * @param time The date/time
	 * @author Dennis Wagelaar
	 */
	public Date getTimeAsDate()
	{
		return this.$calendar.getTime();
	} //getTimeAsDate

	/**
	 * Sets the time attribute.
	 * @param time The date/time
	 * @author Dennis Wagelaar
	 */
	public void setTime(TimeData time)
	{
		this.$calendar = time.$calendar;
		this.$millisec = time.$millisec;
	} //setTime

	/**
	 * Gets the time attribute.
	 * @return The date/time in the format "YYYY-MM-DD HH:MM:SS"
	 * @author Dennis Wagelaar
	 */
	public String getTime()
	{
		return this.$timeFmt.format(this.getTimeAsDate());
	} //getTime

	/**
	 * Gets the time attribute as timestamp.
	 * @return The date/time in the format "YYYYMMDDHHMMSS"
	 * @author Dennis Wagelaar
	 */
	public String getTimestamp()
	{
		return this.$timestampFmt.format(this.getTimeAsDate());
	} //getTimestamp

	/**
	 * Sets the precision attribute.
	 * @param m The amount of milliseconds.
	 * @author Miro Casanova
	 */
	public void setMilliSec(int m)
	{
		this.$millisec = m;
	} //setMilliSec

	/**
	 * Gets the precision attribute.
	 * @return The amount of milliseconds.
	 * @author Miro Casanova
	 */
	public int getMilliSec()
	{
		return this.$millisec;
	} //getMilliSec

	/**
	 * Sets the year part of the date/time.
	 * @param year The year (e.g. 2002)
	 * @author Dennis Wagelaar
	 */
	public void setYear(int year) {
		this.$calendar.set(Calendar.YEAR, year);
	} //setYear

	/**
	 * Gets the year part of the date/time.
	 * @return The year (e.g. 2002)
	 * @author Dennis Wagelaar
	 */
	public int getYear() {
		return this.$calendar.get(Calendar.YEAR);
	} //setYear

	/**
	 * Sets the month part of the date/time.
	 * @param month The month (e.g. 12)
	 * @author Dennis Wagelaar
	 */
	public void setMonth(int month) {
		// month value is 0-based
		this.$calendar.set(Calendar.MONTH, month - 1);
	} //setMonth

	/**
	 * Gets the month part of the date/time.
	 * @return The month (e.g. 12)
	 * @author Dennis Wagelaar
	 */
	public int getMonth() {
		return this.$calendar.get(Calendar.MONTH) + 1;
	} //setMonth

	/**
	 * Sets the day part of the date/time.
	 * @param day The day (e.g. 31)
	 * @author Dennis Wagelaar
	 */
	public void setDay(int day) {
		this.$calendar.set(Calendar.DATE, day);
	} //setDay

	/**
	 * Gets the day part of the date/time.
	 * @return The day (e.g. 31)
	 * @author Dennis Wagelaar
	 */
	public int getDay() {
		return this.$calendar.get(Calendar.DATE);
	} //setDay

	/**
	 * Sets the hour part of the date/time.
	 * @param hour The hour (e.g. 23)
	 * @author Dennis Wagelaar
	 */
	public void setHour(int hour) {
		this.$calendar.set(Calendar.HOUR, hour);
	} //setHour

	/**
	 * Gets the hour part of the date/time.
	 * @return The hour (e.g. 23)
	 * @author Dennis Wagelaar
	 */
	public int getHour() {
		return this.$calendar.get(Calendar.HOUR);
	} //setHour

	/**
	 * Sets the minute part of the date/time.
	 * @param minute The minute (e.g. 59)
	 * @author Dennis Wagelaar
	 */
	public void setMinute(int minute) {
		this.$calendar.set(Calendar.MINUTE, minute);
	} //setMinute

	/**
	 * Gets the minute part of the date/time.
	 * @return The minute (e.g. 59)
	 * @author Dennis Wagelaar
	 */
	public int getMinute() {
		return this.$calendar.get(Calendar.MINUTE);
	} //setMinute

	/**
	 * Sets the second part of the date/time.
	 * @param second The second (e.g. 59)
	 * @author Dennis Wagelaar
	 */
	public void setSecond(int second) {
		this.$calendar.set(Calendar.SECOND, second);
	} //setSecond

	/**
	 * Gets the second part of the date/time.
	 * @return The second (e.g. 59)
	 * @author Dennis Wagelaar
	 */
	public int getSecond() {
		return this.$calendar.get(Calendar.SECOND);
	} //setSecond

} //class TimeData
